@@ -1,5 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import hashlib
 from decimal import Decimal
 from collections import namedtuple, defaultdict
 from itertools import groupby
@@ -9,50 +10,19 @@ from sql.conditionals import Coalesce
 from sql.aggregate import Max, Sum
 
 from trytond.config import config
+from trytond.i18n import gettext
 from trytond.model import Workflow, ModelView, ModelSQL, fields, Check, \
     sequence_ordered, DictSchemaMixin
+from trytond.model.exceptions import AccessError
 from trytond.pyson import Eval, If, Bool
 from trytond.transaction import Transaction
 from trytond.pool import Pool
+from trytond.rpc import RPC
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond.modules.company import CompanyReport
 
-__all__ = ['Statement', 'Line', 'LineGroup',
-    'Origin', 'OriginInformation',
-    'ImportStatementStart', 'ImportStatement',
-    'ReconcileStatement',
-    'StatementReport']
-
-_STATES = {'readonly': Eval('state') != 'draft'}
-_DEPENDS = ['state']
-
-_BALANCE_STATES = _STATES.copy()
-_BALANCE_STATES.update({
-        'invisible': ~Eval('validation', '').in_(['balance']),
-        'required': Eval('validation', '').in_(['balance']),
-        })
-_BALANCE_DEPENDS = _DEPENDS + ['validation']
-
-_AMOUNT_STATES = _STATES.copy()
-_AMOUNT_STATES.update({
-        'invisible': ~Eval('validation', '').in_(['amount']),
-        'required': Eval('validation', '').in_(['amount']),
-        })
-_AMOUNT_DEPENDS = _DEPENDS + ['validation']
-
-_NUMBER_STATES = _STATES.copy()
-_NUMBER_STATES.update({
-        'invisible': ~Eval('validation', '').in_(['number_of_lines']),
-        'required': Eval('validation', '').in_(['number_of_lines']),
-        })
-_NUMBER_DEPENDS = _DEPENDS + ['validation']
-
-STATES = [
-    ('draft', 'Draft'),
-    ('validated', 'Validated'),
-    ('cancel', 'Canceled'),
-    ('posted', 'Posted'),
-    ]
+from .exceptions import (
+    StatementValidateError, StatementValidateWarning, StatementPostError)
 
 if config.getboolean('account_statement', 'filestore', default=False):
     file_id = 'origin_file_id'
@@ -79,13 +49,32 @@ class Unequal(object):
 class Statement(Workflow, ModelSQL, ModelView):
     'Account Statement'
     __name__ = 'account.statement'
+
+    _states = {'readonly': Eval('state') != 'draft'}
+    _depends = ['state']
+    _balance_states = _states.copy()
+    _balance_states.update({
+            'invisible': ~Eval('validation', '').in_(['balance']),
+            'required': Eval('validation', '').in_(['balance']),
+            })
+    _balance_depends = _depends + ['validation']
+    _amount_states = _states.copy()
+    _amount_states.update({
+            'invisible': ~Eval('validation', '').in_(['amount']),
+            'required': Eval('validation', '').in_(['amount']),
+            })
+    _amount_depends = _depends + ['validation']
+    _number_states = _states.copy()
+    _number_states.update({
+            'invisible': ~Eval('validation', '').in_(['number_of_lines']),
+            'required': Eval('validation', '').in_(['number_of_lines']),
+            })
+    _number_depends = _depends + ['validation']
+
     name = fields.Char('Name', required=True)
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True, states=_STATES, domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ],
-        depends=_DEPENDS)
+    company = fields.Many2One(
+        'company.company', "Company", required=True, select=True,
+        states=_states, depends=_depends)
     journal = fields.Many2One('account.statement.journal', 'Journal',
         required=True, select=True,
         domain=[
@@ -95,26 +84,28 @@ class Statement(Workflow, ModelSQL, ModelView):
             'readonly': (Eval('state') != 'draft') | Eval('lines', [0]),
             },
         depends=['state', 'company'])
+    currency = fields.Function(fields.Many2One(
+            'currency.currency', "Currency"), 'on_change_with_currency')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
         'on_change_with_currency_digits')
     date = fields.Date('Date', required=True, select=True)
     start_balance = fields.Numeric('Start Balance',
         digits=(16, Eval('currency_digits', 2)),
-        states=_BALANCE_STATES, depends=_BALANCE_DEPENDS + ['currency_digits'])
+        states=_balance_states, depends=_balance_depends + ['currency_digits'])
     end_balance = fields.Numeric('End Balance',
         digits=(16, Eval('currency_digits', 2)),
-        states=_BALANCE_STATES, depends=_BALANCE_DEPENDS + ['currency_digits'])
+        states=_balance_states, depends=_balance_depends + ['currency_digits'])
     balance = fields.Function(
         fields.Numeric('Balance',
             digits=(16, Eval('currency_digits', 2)),
-            states=_BALANCE_STATES,
-            depends=_BALANCE_DEPENDS + ['currency_digits']),
+            states=_balance_states,
+            depends=_balance_depends + ['currency_digits']),
         'on_change_with_balance')
     total_amount = fields.Numeric('Total Amount',
         digits=(16, Eval('currency_digits', 2)),
-        states=_AMOUNT_STATES, depends=_AMOUNT_DEPENDS + ['currency_digits'])
+        states=_amount_states, depends=_amount_depends + ['currency_digits'])
     number_of_lines = fields.Integer('Number of Lines',
-        states=_NUMBER_STATES, depends=_NUMBER_DEPENDS)
+        states=_number_states, depends=_number_depends)
     lines = fields.One2Many('account.statement.line', 'statement',
         'Lines', states={
             'readonly': (Eval('state') != 'draft') | ~Eval('journal'),
@@ -129,38 +120,36 @@ class Statement(Workflow, ModelSQL, ModelView):
         "Origin File", readonly=True,
         file_id=file_id, store_prefix=store_prefix)
     origin_file_id = fields.Char("Origin File ID", readonly=True)
-    state = fields.Selection(STATES, 'State', readonly=True, select=True)
+    state = fields.Selection([
+            ('draft', "Draft"),
+            ('validated', "Validated"),
+            ('cancelled', "Cancelled"),
+            ('posted', "Posted"),
+            ], "State", readonly=True, select=True)
     validation = fields.Function(fields.Char('Validation'),
         'on_change_with_validation')
     to_reconcile = fields.Function(
         fields.Boolean("To Reconcile"), 'get_to_reconcile')
 
+    del _states, _depends
+    del _balance_states, _balance_depends
+    del _amount_states, _amount_depends
+    del _number_states, _number_depends
+
     @classmethod
     def __setup__(cls):
         super(Statement, cls).__setup__()
         cls._order[0] = ('id', 'DESC')
-        cls._error_messages.update({
-                'wrong_end_balance': 'End Balance must be "%s".',
-                'wrong_total_amount': 'Total Amount must be "%s".',
-                'wrong_number_of_lines': 'Number of Lines must be "%s".',
-                'delete_cancel': ('Statement "%s" must be cancelled before '
-                    'deletion.'),
-                'paid_invoice_draft_statement': ('There are paid invoices on '
-                    'draft statements.'),
-                'post_with_pending_amount': ('Origin line "%(origin)s" '
-                    'of statement "%(statement)s" still has a pending amount '
-                    'of "%(amount)s".'),
-                })
         cls._transitions |= set((
                 ('draft', 'validated'),
-                ('draft', 'cancel'),
+                ('draft', 'cancelled'),
                 ('validated', 'posted'),
-                ('validated', 'cancel'),
-                ('cancel', 'draft'),
+                ('validated', 'cancelled'),
+                ('cancelled', 'draft'),
                 ))
         cls._buttons.update({
                 'draft': {
-                    'invisible': Eval('state') != 'cancel',
+                    'invisible': Eval('state') != 'cancelled',
                     'depends': ['state'],
                     },
                 'validate_statement': {
@@ -176,11 +165,15 @@ class Statement(Workflow, ModelSQL, ModelView):
                     'depends': ['state'],
                     },
                 'reconcile': {
-                    'invisible': Eval('state').in_(['draft', 'cancel']),
+                    'invisible': Eval('state').in_(['draft', 'cancelled']),
                     'readonly': ~Eval('to_reconcile'),
                     'depends': ['state', 'to_reconcile'],
                     },
                 })
+        cls.__rpc__.update({
+                'post': RPC(
+                    readonly=False, instantiate=0, fresh_session=True),
+                })
 
     @classmethod
     def __register__(cls, module_name):
@@ -200,6 +193,11 @@ class Statement(Workflow, ModelSQL, ModelView):
                 [sql_table.id.cast(cls.name.sql_type().base)],
                 where=sql_table.name == Null))
 
+        # Migration from 5.6: rename state cancel to cancelled
+        cursor.execute(*sql_table.update(
+                [sql_table.state], ['cancelled'],
+                where=sql_table.state == 'cancel'))
+
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
@@ -238,6 +236,11 @@ class Statement(Workflow, ModelSQL, ModelView):
         statement, = statements
         self.start_balance = statement.end_balance
 
+    @fields.depends('journal')
+    def on_change_with_currency(self, name=None):
+        if self.journal:
+            return self.journal.currency.id
+
     @fields.depends('journal')
     def on_change_with_currency_digits(self, name=None):
         if self.journal:
@@ -255,6 +258,51 @@ class Statement(Workflow, ModelSQL, ModelView):
         return ((getattr(self, 'end_balance', 0) or 0)
             - (getattr(self, 'start_balance', 0) or 0))
 
+    @fields.depends('origins', 'lines', 'journal', 'company')
+    def on_change_origins(self):
+        if not self.journal or not self.origins or not self.company:
+            return
+        if self.journal.currency != self.company.currency:
+            return
+
+        invoices = set()
+        for line in self.lines:
+            if (getattr(line, 'invoice', None)
+                    and line.invoice.currency == self.company.currency):
+                invoices.add(line.invoice)
+        for origin in self.origins:
+            for line in origin.lines:
+                if (getattr(line, 'invoice', None)
+                        and line.invoice.currency == self.company.currency):
+                    invoices.add(line.invoice)
+        invoice_id2amount_to_pay = {}
+        for invoice in invoices:
+            if invoice.type == 'out':
+                sign = -1
+            else:
+                sign = 1
+            invoice_id2amount_to_pay[invoice.id] = sign * invoice.amount_to_pay
+
+        origins = list(self.origins)
+        for origin in origins:
+            lines = list(origin.lines)
+            for line in lines:
+                if (getattr(line, 'invoice', None) and line.id
+                        and line.invoice.id in invoice_id2amount_to_pay):
+                    amount_to_pay = invoice_id2amount_to_pay[line.invoice.id]
+                    if (amount_to_pay
+                            and getattr(line, 'amount', None)
+                            and (line.amount >= 0) == (amount_to_pay <= 0)):
+                        if abs(line.amount) > abs(amount_to_pay):
+                            line.amount = amount_to_pay.copy_sign(line.amount)
+                        else:
+                            invoice_id2amount_to_pay[line.invoice.id] = (
+                                line.amount + amount_to_pay)
+                    else:
+                        line.invoice = None
+            origin.lines = lines
+        self.origins = origins
+
     @fields.depends('lines', 'journal', 'company')
     def on_change_lines(self):
         pool = Pool()
@@ -371,13 +419,21 @@ class Statement(Workflow, ModelSQL, ModelView):
         for key, lines in groupby(lines, key=self._group_key):
             yield Line(**dict(key + (('lines', list(lines)),)))
 
+    @classmethod
+    def view_attributes(cls):
+        return super().view_attributes() + [
+            ('/tree', 'visual', If(Eval('state') == 'cancelled', 'muted', '')),
+            ]
+
     @classmethod
     def delete(cls, statements):
         # Cancel before delete
         cls.cancel(statements)
         for statement in statements:
-            if statement.state != 'cancel':
-                cls.raise_user_error('delete_cancel', (statement.rec_name,))
+            if statement.state != 'cancelled':
+                raise AccessError(
+                    gettext('account_statement.msg_statement_delete_cancel',
+                        statement=statement.rec_name))
         super(Statement, cls).delete(statements)
 
     @classmethod
@@ -390,12 +446,18 @@ class Statement(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Lang = pool.get('ir.lang')
 
-        end_balance = (self.start_balance
+        amount = (self.start_balance
             + sum(l.amount for l in self.lines))
-        if end_balance != self.end_balance:
+        if amount != self.end_balance:
             lang = Lang.get()
-            amount = lang.currency(end_balance, self.journal.currency)
-            self.raise_user_error('wrong_end_balance', amount)
+            end_balance = lang.currency(
+                self.end_balance, self.journal.currency)
+            amount = lang.currency(amount, self.journal.currency)
+            raise StatementValidateError(
+                gettext('account_statement.msg_statement_wrong_end_balance',
+                    statement=self.rec_name,
+                    end_balance=end_balance,
+                    amount=amount))
 
     def validate_amount(self):
         pool = Pool()
@@ -404,13 +466,29 @@ class Statement(Workflow, ModelSQL, ModelView):
         amount = sum(l.amount for l in self.lines)
         if amount != self.total_amount:
             lang = Lang.get()
+            total_amount = lang.currency(
+                self.total_amount, self.journal.currency)
             amount = lang.currency(amount, self.journal.currency)
-            self.raise_user_error('wrong_total_amount', amount)
+            raise StatementValidateError(
+                gettext('account_statement.msg_statement_wrong_total_amount',
+                    statement=self.rec_name,
+                    total_amount=total_amount,
+                    amount=amount))
 
     def validate_number_of_lines(self):
         number = len(list(self.grouped_lines))
-        if number != self.number_of_lines:
-            self.raise_user_error('wrong_number_of_lines', number)
+        if number > self.number_of_lines:
+            raise StatementValidateError(
+                gettext('account_statement'
+                    '.msg_statement_wrong_number_of_lines_remove',
+                    statement=self.rec_name,
+                    n=number - self.number_of_lines))
+        elif number < self.number_of_lines:
+            raise StatementValidateError(
+                gettext('account_statement'
+                    '.msg_statement_wrong_number_of_lines_remove',
+                    statement=self.rec_name,
+                    n=self.number_of_lines - number))
 
     @classmethod
     @ModelView.button
@@ -418,22 +496,41 @@ class Statement(Workflow, ModelSQL, ModelView):
     def validate_statement(cls, statements):
         pool = Pool()
         Line = pool.get('account.statement.line')
-
+        Warning = pool.get('res.user.warning')
+        paid_cancelled_invoice_lines = []
         for statement in statements:
             getattr(statement, 'validate_%s' % statement.validation)()
+            paid_cancelled_invoice_lines.extend(l for l in statement.lines
+                if l.invoice and l.invoice.state in {'cancelled', 'paid'})
+
+        if paid_cancelled_invoice_lines:
+            warning_key = ('%s.statement_paid_cancelled_invoice_lines' %
+                    hashlib.md5(str(paid_cancelled_invoice_lines).encode(
+                        'utf-8')).hexdigest())
+            if Warning.check(warning_key):
+                raise StatementValidateWarning(warning_key,
+                    gettext('account_statement'
+                        '.msg_statement_invoice_paid_cancelled'))
+            Line.write(paid_cancelled_invoice_lines, {
+                    'invoice': None,
+                    })
 
         cls.create_move(statements)
 
         cls.write(statements, {
                 'state': 'validated',
                 })
-        common_lines = Line.search([
+        common_lines = [l for l in Line.search([
                 ('statement.state', '=', 'draft'),
-                ('invoice.state', '=', 'paid'),
+                ('invoice.state', 'in', ['posted', 'paid']),
                 ])
+            if l.invoice.reconciled]
         if common_lines:
             warning_key = '_'.join(str(l.id) for l in common_lines)
-            cls.raise_user_warning(warning_key, 'paid_invoice_draft_statement')
+            if Warning.check(warning_key):
+                raise StatementValidateWarning(warning_key,
+                    gettext('account_statement'
+                        '.msg_statement_paid_invoice_draft'))
             Line.write(common_lines, {
                     'invoice': None,
                     })
@@ -537,22 +634,29 @@ class Statement(Workflow, ModelSQL, ModelView):
     @ModelView.button
     @Workflow.transition('posted')
     def post(cls, statements):
-        StatementLine = Pool().get('account.statement.line')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        StatementLine = pool.get('account.statement.line')
         for statement in statements:
             for origin in statement.origins:
                 if origin.pending_amount:
-                    cls.raise_user_error('post_with_pending_amount', {
-                            'origin': origin.rec_name,
-                            'amount': origin.pending_amount,
-                            'statement': statement.rec_name,
-                            })
-
+                    lang = Lang.get()
+                    amount = lang.currency(
+                        origin.pending_amount, statement.journal.currency)
+                    raise StatementPostError(
+                        gettext('account_statement'
+                            '.msg_statement_post_pending_amount',
+                            statement=statement.rec_name,
+                            amount=amount,
+                            origin=origin.rec_name))
+        # Write state to skip statement test on Move.post
+        cls.write(statements, {'state': 'posted'})
         lines = [l for s in statements for l in s.lines]
         StatementLine.post_move(lines)
 
     @classmethod
     @ModelView.button
-    @Workflow.transition('cancel')
+    @Workflow.transition('cancelled')
     def cancel(cls, statements):
         StatementLine = Pool().get('account.statement.line')
 
@@ -567,6 +671,7 @@ class Statement(Workflow, ModelSQL, ModelView):
 
 def origin_mixin(_states, _depends):
     class Mixin:
+        __slots__ = ()
         statement = fields.Many2One(
             'account.statement', "Statement",
             required=True, ondelete='CASCADE', states=_states,
@@ -582,16 +687,24 @@ def origin_mixin(_states, _depends):
             "Date", required=True, states=_states, depends=_depends)
         amount = fields.Numeric(
             "Amount", required=True,
-            digits=(16, Eval('_parent_statement', {})
-                .get('currency_digits', 2)),
-            states=_states, depends=_depends)
+            digits=(16, Eval('currency_digits', 2)),
+            states=_states, depends=_depends + ['currency_digits'])
+        currency = fields.Function(fields.Many2One(
+                'currency.currency', "Currency"), 'on_change_with_currency')
+        currency_digits = fields.Function(fields.Integer(
+                "Currency Digits"), 'on_change_with_currency_digits')
         party = fields.Many2One(
-            'party.party', "Party", states=_states, depends=_depends)
+            'party.party', "Party", states=_states,
+            context={
+                'company': Eval('company', -1),
+                },
+            depends=_depends + ['company'])
         account = fields.Many2One(
             'account.account', "Account",
             domain=[
                 ('company', '=', Eval('company', 0)),
-                ('kind', '!=', 'view'),
+                ('type', '!=', None),
+                ('closed', '!=', True),
                 ],
             context={
                 'date': Eval('date'),
@@ -600,6 +713,11 @@ def origin_mixin(_states, _depends):
         description = fields.Char(
             "Description", states=_states, depends=_depends)
 
+        @classmethod
+        def __setup__(cls):
+            super().__setup__()
+            cls.__access__.add('statement')
+
         @classmethod
         def get_statement_states(cls):
             pool = Pool()
@@ -620,6 +738,16 @@ def origin_mixin(_states, _depends):
         def search_company(cls, name, clause):
             return [('statement.' + clause[0],) + tuple(clause[1:])]
 
+        @fields.depends('statement', '_parent_statement.journal')
+        def on_change_with_currency(self, name=None):
+            if self.statement and self.statement.journal:
+                return self.statement.journal.currency.id
+
+        @fields.depends('statement', '_parent_statement.journal')
+        def on_change_with_currency_digits(self, name=None):
+            if self.statement and self.statement.journal:
+                return self.statement.journal.currency.digits
+
     return Mixin
 
 
@@ -649,6 +777,7 @@ class Line(
                 ('state', '!=', '')),
             ],
         states=_states,
+        context={'with_payment': False},
         depends=['party', 'account'] + _depends)
     origin = fields.Many2One('account.statement.origin', 'Origin',
         readonly=True,
@@ -666,15 +795,15 @@ class Line(
         super(Line, cls).__setup__()
         if 'origin' not in cls.date.depends:
             cls.date.states.update({
-                    'readonly': (cls.date.states['readonly'] |
-                        Bool(Eval('origin', 0))),
+                    'readonly': (cls.date.states['readonly']
+                        | Bool(Eval('origin', 0))),
                     })
             cls.date.depends.append('origin')
         cls.account.required = True
         t = cls.__table__()
         cls._sql_constraints += [
             ('check_statement_line_amount', Check(t, t.amount != 0),
-                'Amount should be a positive or negative value.'),
+                'account_statement.msg_line_amount_non_zero'),
             ]
 
     @staticmethod
@@ -772,7 +901,7 @@ class Line(
             return self.origin.company.id
         return company
 
-    @fields.depends('origin', '_parent_origin.state')
+    @fields.depends('origin', '_parent_origin.statement_state')
     def on_change_with_statement_state(self, name=None):
         try:
             state = super(Line, self).on_change_with_statement_state()
@@ -817,6 +946,8 @@ class Line(
             if line.invoice in invoice_payments:
                 Invoice.add_payment_lines(invoice_payments)
                 invoice_payments.clear()
+                MoveLine.reconcile(*to_reconcile)
+                to_reconcile.clear()
 
             reconcile_lines = line.invoice.get_reconcile_lines_for_amount(
                 move_line.credit - move_line.debit)
@@ -834,7 +965,8 @@ class Line(
     @classmethod
     def post_move(cls, lines):
         Move = Pool().get('account.move')
-        Move.post(list({l.move for l in lines if l.move}))
+        Move.post(list({l.move for l in lines
+                    if l.move and l.move.state != 'posted'}))
 
     @classmethod
     def delete_move(cls, lines):
@@ -867,6 +999,7 @@ class Line(
             second_currency = None
 
         return MoveLine(
+            origin=self,
             description=self.description,
             debit=amount < zero and -amount or zero,
             credit=amount >= zero and amount or zero,
@@ -876,6 +1009,7 @@ class Line(
             amount_second_currency=amount_second_currency,
             )
 
+
 del _states, _depends
 
 
@@ -901,6 +1035,7 @@ class LineGroup(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(LineGroup, cls).__setup__()
+        cls.__access__.add('statement')
         cls._order.insert(0, ('date', 'DESC'))
 
     @classmethod
@@ -966,8 +1101,8 @@ class Origin(origin_mixin(_states, _depends), ModelSQL, ModelView):
     lines = fields.One2Many(
         'account.statement.line', 'origin', "Lines",
         states={
-            'readonly': ((Eval('statement_id', -1) < 0) |
-                ~Eval('statement_state').in_(['draft', 'validated'])),
+            'readonly': ((Eval('statement_id', -1) < 0)
+                | ~Eval('statement_state').in_(['draft', 'validated'])),
             },
         domain=[
             ('statement', '=', Eval('statement')),
@@ -981,8 +1116,17 @@ class Origin(origin_mixin(_states, _depends), ModelSQL, ModelView):
             digits=(16, Eval('_parent_statement', {})
                 .get('currency_digits', 2))),
         'on_change_with_pending_amount', searcher='search_pending_amount')
-    informations = fields.Dict(
-        'account.statement.origin.information', "Informations", readonly=True)
+    information = fields.Dict(
+        'account.statement.origin.information', "Information", readonly=True)
+
+    @classmethod
+    def __register__(cls, module_name):
+        table = cls.__table_handler__(module_name)
+
+        # Migration from 5.0: rename informations into information
+        table.column_rename('informations', 'information')
+
+        super(Origin, cls).__register__(module_name)
 
     @fields.depends('statement', '_parent_statement.id')
     def on_change_with_statement_id(self, name=None):
@@ -1012,6 +1156,8 @@ class Origin(origin_mixin(_states, _depends), ModelSQL, ModelView):
                     table.amount - Coalesce(Sum(line.amount), 0), value),
                 group_by=table.id))
         return [('id', 'in', query)]
+
+
 del _states, _depends
 
 
@@ -1066,11 +1212,8 @@ class ReconcileStatement(Wizard):
     start = StateAction('account.act_reconcile')
 
     def do_start(self, action):
-        pool = Pool()
-        Statement = pool.get('account.statement')
-        statements = Statement.browse(Transaction().context['active_ids'])
-        lines = sum(([int(l) for l in s.lines_to_reconcile]
-                for s in statements), [])
+        lines = sum(
+            ([int(l) for l in s.lines_to_reconcile] for s in self.records), [])
         return action, {
             'model': 'account.move.line',
             'ids': lines,
